@@ -27,7 +27,7 @@
 #include <libxml/parser.h>
 #include "common.c"
 
-#define MIN(a,b) ((a > b) ? b : a)
+#define MIN(a, b) ((a > b) ? b : a)
 
 // entities to check against:
 // perl -lne'/type="(IAcad.*?)" / and print $1' test/test-data/*/*.xml|sort -u
@@ -46,7 +46,7 @@
     IAcadSpline
     IAcadXline
  */
-int load_dwg (char *dwgfilename, xmlNodePtr rootnode);
+int xml_dwg (char *dwgfilename, xmlNodePtr rootnode);
 void common_entity_attrs (xmlNodePtr node, const Dwg_Object *obj);
 void add_2dpolyline (xmlNodePtr rootnode, const Dwg_Object *obj);
 void add_3dpolyline (xmlNodePtr rootnode, const Dwg_Object *obj);
@@ -64,6 +64,7 @@ void add_ray (xmlNodePtr rootnode, const Dwg_Object *obj);
 void add_spline (xmlNodePtr rootnode, const Dwg_Object *obj);
 void add_table (xmlNodePtr rootnode, const Dwg_Object *obj);
 void add_text (xmlNodePtr rootnode, const Dwg_Object *obj);
+void add_mtext (xmlNodePtr rootnode, const Dwg_Object *obj);
 void add_xline (xmlNodePtr rootnode, const Dwg_Object *obj);
 
 #define newXMLProp(name, buf)                                                 \
@@ -95,7 +96,8 @@ common_entity_attrs (xmlNodePtr node, const Dwg_Object *obj)
   dwg_point_3d extrusion;
   double thickness;
 
-  // TODO: EntityTransparency, ObjectID, ObjectID32, Visible, TrueColor
+  // TODO: EntityTransparency (ByLayer), ObjectID=int, ObjectID32=int, Visible,
+  // TrueColor OwnerID=int OwnerID32=int TrueColor Document=dwg ptr
 
   buf = doubletohex (obj->handle.value);
   newXMLProp ("Handle", buf);
@@ -146,7 +148,8 @@ common_entity_attrs (xmlNodePtr node, const Dwg_Object *obj)
      Ellipse, ExternalReference, Hatch, Leader, LightweightPolyline,
      Line, MInsertBlock, MText, Point, Polyline, Region, Section,
      Shape, Solid, Text, Tolerance, Trace */
-  if (dwg_dynapi_entity_value (ent->tio.LINE, obj->name, "extrusion", &extrusion, NULL))
+  if (dwg_dynapi_entity_value (ent->tio.LINE, obj->name, "extrusion",
+                               &extrusion, NULL))
     {
       buf = spointprepare (extrusion.x, extrusion.y, extrusion.z);
       newXMLProp ("Normal", buf);
@@ -154,7 +157,8 @@ common_entity_attrs (xmlNodePtr node, const Dwg_Object *obj)
 
   /* Arc, Attribute, AttributeReference, Circle, LightweightPolyline,
      Line, Point, Polyline, Shape, Solid, Text, Trace */
-  if (dwg_dynapi_entity_value (ent->tio.LINE, obj->name, "thickness", &thickness, NULL))
+  if (dwg_dynapi_entity_value (ent->tio.LINE, obj->name, "thickness",
+                               &thickness, NULL))
     {
       buf = doubletochar (thickness);
       newXMLProp ("Thickness", buf);
@@ -175,6 +179,7 @@ add_line (xmlNodePtr rootnode, const Dwg_Object *obj)
 
   newXMLcProp ("type", "IAcadLine");
   newXMLcProp ("desc", "IAcadLine: AutoCAD Line Interface");
+  newXMLcProp ("ObjectName", "AcDbLine");
 
   buf = spointprepare (line->end.x, line->end.y, line->end.z);
   newXMLProp ("EndPoint", buf);
@@ -200,6 +205,7 @@ add_circle (xmlNodePtr rootnode, const Dwg_Object *obj)
 
   newXMLcProp ("type", "IAcadCircle");
   newXMLcProp ("desc", "IAcadCircle: AutoCAD Circle Interface");
+  newXMLcProp ("ObjectName", "AcDbCircle");
 
   buf = spointprepare (circle->center.x, circle->center.y, circle->center.z);
   newXMLProp ("Center", buf);
@@ -233,14 +239,18 @@ add_helix (xmlNodePtr rootnode, const Dwg_Object *obj)
 
   newXMLcProp ("type", "IAcadHelix");
   newXMLcProp ("desc", "IAcadHelix: IAcadSpring Interface");
+  newXMLcProp ("ObjectName", "AcDbHelix");
 
-  buf = spointprepare (helix->start_pt.x, helix->start_pt.y, helix->start_pt.z);
+  buf = spointprepare (helix->start_pt.x, helix->start_pt.y,
+                       helix->start_pt.z);
   newXMLProp ("Position", buf);
 
-  buf = spointprepare (helix->axis_base_pt.x, helix->axis_base_pt.y, helix->axis_base_pt.z);
+  buf = spointprepare (helix->axis_base_pt.x, helix->axis_base_pt.y,
+                       helix->axis_base_pt.z);
   newXMLProp ("AxisPoint", buf);
 
-  buf = spointprepare (helix->axis_vector.x, helix->axis_vector.y, helix->axis_vector.z);
+  buf = spointprepare (helix->axis_vector.x, helix->axis_vector.y,
+                       helix->axis_vector.z);
   newXMLProp ("AxisVector", buf);
 
   dtostring = doubletochar (helix->turn_height);
@@ -292,6 +302,7 @@ add_lwpolyline (xmlNodePtr rootnode, const Dwg_Object *obj)
   newXMLcProp ("type", "IAcadLWPolyline");
   newXMLcProp ("desc",
                "IAcadLWPolyline: AutoCAD Lightweight Polyline Interface");
+  newXMLcProp ("ObjectName", "AcDbPolyline");
 
   newXMLcProp ("Closed", lwpline->flag & 1 ? "1" : "0");
   // ConstantWidth="0.0"
@@ -302,7 +313,7 @@ add_lwpolyline (xmlNodePtr rootnode, const Dwg_Object *obj)
       sprintf ((char *)buf, "(%.4f %.4f %.4f %.4f %.4f %.4f ... )", pts[0].x,
                pts[0].y, pts[1].x, pts[1].y, pts[2].x, pts[2].y);
       newXMLProp ("Coordinates", buf);
-      free (buf);
+      // free (buf);
     }
   free (pts);
 
@@ -315,8 +326,6 @@ add_lwpolyline (xmlNodePtr rootnode, const Dwg_Object *obj)
   newXMLProp ("Normal", buf);
   */
 
-  newXMLcProp ("ObjectName", "AcDbPolyline");
-
   /* See common_entity_attrs
   buf = doubletochar (lwpline->thickness);
   newXMLProp ("Thickness", buf);
@@ -345,6 +354,7 @@ add_2dpolyline (xmlNodePtr rootnode, const Dwg_Object *obj)
 
   newXMLcProp ("type", "IAcad2DPolyline");
   newXMLcProp ("desc", "IAcad2DPolyline: AutoCAD 2dPolyline Interface");
+  newXMLcProp ("ObjectName", "AcDb2dPolyline");
 
   newXMLcProp ("Closed", pline->flag & 512 ? "1" : "0");
 
@@ -383,6 +393,7 @@ add_3dpolyline (xmlNodePtr rootnode, const Dwg_Object *obj)
 
   newXMLcProp ("type", "IAcad3DPolyline");
   newXMLcProp ("desc", "IAcad3DPolyline: AutoCAD 3dPolyline Interface");
+  newXMLcProp ("ObjectName", "AcDb3dPolyline");
 
   newXMLcProp ("Closed", pline->flag & 512 ? "1" : "0");
 
@@ -396,7 +407,6 @@ add_3dpolyline (xmlNodePtr rootnode, const Dwg_Object *obj)
   free (pts);
 
   common_entity_attrs (node, obj);
-  newXMLcProp ("ObjectName", "Acad3DPolyline");
   xmlAddChild (rootnode, node);
 }
 
@@ -416,6 +426,7 @@ add_arc (xmlNodePtr rootnode, const Dwg_Object *obj)
 
   newXMLcProp ("type", "IAcadArc");
   newXMLcProp ("desc", "IAcadArc: AutoCAD Arc Interface");
+  newXMLcProp ("ObjectName", "AcDbArc");
 
   buf = spointprepare (arc->center.x, arc->center.y, arc->center.z);
   newXMLProp ("Center", buf);
@@ -455,6 +466,7 @@ add_block (xmlNodePtr rootnode, const Dwg_Object *obj)
 
       newXMLcProp ("type", "IAcadBlock");
       newXMLcProp ("desc", "IAcadBlock: AutoCAD Block Interface");
+      newXMLcProp ("ObjectName", "AcDbBlock");
 
       newXMLcProp ("EffectiveName", block->name);
 
@@ -511,6 +523,7 @@ add_ellipse (xmlNodePtr rootnode, const Dwg_Object *obj)
   // Now the attributes
   newXMLcProp ("type", "IAcadEllipse");
   newXMLcProp ("desc", "IAcadEllipse: AutoCAD Ellipse Interface");
+  newXMLcProp ("ObjectName", "AcDbEllipse");
 
   buf = spointprepare (ellipse->center.x, ellipse->center.y,
                        ellipse->center.z);
@@ -549,6 +562,7 @@ add_mline (xmlNodePtr rootnode, const Dwg_Object *obj)
 
   newXMLcProp ("type", "IAcadMLine");
   newXMLcProp ("desc", "IAcadMLine: IAcadMLine Interface");
+  newXMLcProp ("ObjectName", "AcDbMline");
 
   if (mline->num_verts >= 3)
     {
@@ -587,6 +601,7 @@ add_xline (xmlNodePtr rootnode, const Dwg_Object *obj)
 
   newXMLcProp ("type", "IAcadXline");
   newXMLcProp ("desc", "IAcadXline: AutoCAD Xline Interface");
+  newXMLcProp ("ObjectName", "AcDbXline");
 
   buf = spointprepare (line->point.x, line->point.y, line->point.z);
   newXMLProp ("BasePoint", buf);
@@ -617,6 +632,7 @@ add_point (xmlNodePtr rootnode, const Dwg_Object *obj)
 
   newXMLcProp ("type", "IAcadPoint");
   newXMLcProp ("desc", "IAcadPoint: AutoCAD Point Interface");
+  newXMLcProp ("ObjectName", "AcDbPoint");
 
   buf = spointprepare (point->x, point->y, point->z);
   newXMLProp ("Coordinates", buf);
@@ -644,6 +660,7 @@ add_ray (xmlNodePtr rootnode, const Dwg_Object *obj)
 
   newXMLcProp ("type", "IAcadRay");
   newXMLcProp ("desc", "IAcadRay: AutoCAD Ray Interface");
+  newXMLcProp ("ObjectName", "AcDbRay");
 
   buf = spointprepare (ray->point.x, ray->point.y, ray->point.z);
   newXMLProp ("BasePoint", buf);
@@ -670,6 +687,7 @@ add_spline (xmlNodePtr rootnode, const Dwg_Object *obj)
 
   newXMLcProp ("type", "IAcadSpline");
   newXMLcProp ("desc", "IAcadSpline: AutoCAD Spline Interface");
+  newXMLcProp ("ObjectName", "AcDbSpline");
 
   newXMLcProp ("Closed", spline->closed_b ? "1" : "0");
   newXMLProp ("Degree", doubletochar (spline->degree));
@@ -693,7 +711,7 @@ add_spline (xmlNodePtr rootnode, const Dwg_Object *obj)
       buf[0] = '(';
       buf[1] = '\0';
       len = 1;
-      for (unsigned i = 0; i < MIN(6,spline->num_knots); i++)
+      for (unsigned i = 0; i < MIN (6, spline->num_knots); i++)
         {
           // Knots="(0.0 0.0 0.0 0.0 12.2776 26.0835 ... )"
           char buf1[24];
@@ -706,18 +724,18 @@ add_spline (xmlNodePtr rootnode, const Dwg_Object *obj)
               bufsize = len + 24;
               buf = realloc (buf, bufsize);
             }
-          strncat ((char*)buf, buf1, bufsize - len);
+          strncat ((char *)buf, buf1, bufsize - len);
         }
       if (spline->num_knots > 6)
-        strcat ((char*)buf, " ... ");
-      strcat ((char*)buf, ")");
+        strcat ((char *)buf, " ... ");
+      strcat ((char *)buf, ")");
       newXMLProp ("Knots", buf);
     }
   newXMLProp ("KnotParameterization", inttochar (spline->knotparam));
   newXMLProp ("NumberOfControlPoints", inttochar (spline->num_ctrl_pts));
   newXMLProp ("NumberOfFitPoints", inttochar (spline->num_fit_pts));
-  newXMLcProp ("SplineFrame", spline->splineflags1 & 2 ? "1" : "0");
-  newXMLcProp ("SplineMethod", spline->splineflags1 & 1 ? "1" : "0");
+  newXMLcProp ("SplineFrame", spline->splineflags & 2 ? "1" : "0");
+  newXMLcProp ("SplineMethod", spline->splineflags & 1 ? "1" : "0");
 
   if (spline->num_ctrl_pts)
     {
@@ -726,7 +744,7 @@ add_spline (xmlNodePtr rootnode, const Dwg_Object *obj)
       buf[0] = '(';
       buf[1] = '\0';
       len = 1;
-      for (unsigned i = 0; i < MIN(2,spline->num_ctrl_pts); i++)
+      for (unsigned i = 0; i < MIN (2, spline->num_ctrl_pts); i++)
         {
           // ControlPoints="(0.0 0.0 0.0 0.0  12.2776 26.0835 ... )"
           char buf1[24];
@@ -734,13 +752,13 @@ add_spline (xmlNodePtr rootnode, const Dwg_Object *obj)
             len += snprintf (buf1, 24, "%f", spline->ctrl_pts[i].x);
           else
             len += snprintf (buf1, 24, " %f", spline->ctrl_pts[i].x);
-          strncat ((char*)buf, buf1, bufsize - len);
+          strncat ((char *)buf, buf1, bufsize - len);
           len += snprintf (buf1, 24, " %f", spline->ctrl_pts[i].y);
-          strncat ((char*)buf, buf1, bufsize - len);
+          strncat ((char *)buf, buf1, bufsize - len);
           len += snprintf (buf1, 24, " %f", spline->ctrl_pts[i].z);
-          strncat ((char*)buf, buf1, bufsize - len);
+          strncat ((char *)buf, buf1, bufsize - len);
           len += snprintf (buf1, 24, " %f ", spline->ctrl_pts[i].w);
-          strncat ((char*)buf, buf1, bufsize - len);
+          strncat ((char *)buf, buf1, bufsize - len);
 
           if (len > bufsize)
             {
@@ -749,8 +767,8 @@ add_spline (xmlNodePtr rootnode, const Dwg_Object *obj)
             }
         }
       if (spline->num_ctrl_pts > 2)
-        strcat ((char*)buf, " ... ");
-      strcat ((char*)buf, ")");
+        strcat ((char *)buf, " ... ");
+      strcat ((char *)buf, ")");
       newXMLProp ("ControlPoints", buf);
     }
 
@@ -761,18 +779,18 @@ add_spline (xmlNodePtr rootnode, const Dwg_Object *obj)
       buf[0] = '(';
       buf[1] = '\0';
       len = 1;
-      for (unsigned i = 0; i < MIN(2,spline->num_fit_pts); i++)
+      for (unsigned i = 0; i < MIN (2, spline->num_fit_pts); i++)
         {
           char buf1[24];
           if (!i)
             len += snprintf (buf1, 24, "%f", spline->fit_pts[i].x);
           else
             len += snprintf (buf1, 24, " %f", spline->fit_pts[i].x);
-          strncat ((char*)buf, buf1, bufsize - len);
+          strncat ((char *)buf, buf1, bufsize - len);
           len += snprintf (buf1, 24, " %f", spline->fit_pts[i].y);
-          strncat ((char*)buf, buf1, bufsize - len);
+          strncat ((char *)buf, buf1, bufsize - len);
           len += snprintf (buf1, 24, " %f", spline->fit_pts[i].z);
-          strncat ((char*)buf, buf1, bufsize - len);
+          strncat ((char *)buf, buf1, bufsize - len);
 
           if (len > bufsize)
             {
@@ -781,11 +799,11 @@ add_spline (xmlNodePtr rootnode, const Dwg_Object *obj)
             }
         }
       if (spline->num_fit_pts > 2)
-        strcat ((char*)buf, " ... ");
-      strcat ((char*)buf, ")");
+        strcat ((char *)buf, " ... ");
+      strcat ((char *)buf, ")");
       newXMLProp ("FitPoints", buf);
     }
-  
+
   common_entity_attrs (node, obj);
   xmlAddChild (rootnode, node);
 }
@@ -798,12 +816,16 @@ add_spline (xmlNodePtr rootnode, const Dwg_Object *obj)
 void
 add_text (xmlNodePtr rootnode, const Dwg_Object *obj)
 {
+  const Dwg_Version_Type dwg_version = obj->parent->header.from_version;
   Dwg_Entity_TEXT *text = obj->tio.entity->tio.TEXT;
   xmlChar *buf, *dtostring;
   xmlNodePtr node = newXMLEntity (rootnode);
+  char *name;
+  int error;
 
   newXMLcProp ("type", "IAcadText");
   newXMLcProp ("desc", "IAcadText: AutoCAD Text Interface");
+  newXMLcProp ("ObjectName", "AcDbText");
 
   dtostring = doubletochar (text->height);
   newXMLProp ("Height", dtostring);
@@ -811,12 +833,90 @@ add_text (xmlNodePtr rootnode, const Dwg_Object *obj)
   buf = spointprepare2 (text->ins_pt.x, text->ins_pt.y);
   newXMLProp ("InsertionPoint", buf);
 
+  name = dwg_ref_get_table_name (text->style, &error);
+  if (!error)
+    {
+      newXMLcProp ("StyleName", name);
+      if (dwg_version >= R_2007)
+        free (name);
+    }
+
   newXMLcProp ("TextString", (xmlChar *)text->text_value);
 
   dtostring = doubletochar (text->width_factor);
   newXMLProp ("Width", dtostring);
 
-  //@TODO: Lots of attributes were also left. Check this.
+  //@TODO: Lots of attributes were also left. Check MText also
+
+  common_entity_attrs (node, obj);
+  xmlAddChild (rootnode, node);
+}
+
+/*
+ * This function emits all the multi-line text related attributes
+ * @param xmlNodePtr rootnode The rootnode of the XML Document
+ * @param Dwg_Object The DWG Object
+ */
+void
+add_mtext (xmlNodePtr rootnode, const Dwg_Object *obj)
+{
+  const Dwg_Version_Type dwg_version = obj->parent->header.from_version;
+  Dwg_Entity_MTEXT *text = obj->tio.entity->tio.MTEXT;
+  xmlChar *buf, *dtostring;
+  xmlNodePtr node = newXMLEntity (rootnode);
+  // double rotation;
+  char *name;
+  int error;
+
+  newXMLcProp ("type", "IAcadMText");
+  newXMLcProp ("desc", "IAcadMText: AutoCAD MText Interface");
+  newXMLcProp ("ObjectName", "AcDbMText");
+
+  dtostring = doubletochar (text->rect_height);
+  newXMLProp ("Height", dtostring);
+
+  buf = spointprepare2 (text->ins_pt.x, text->ins_pt.y);
+  newXMLProp ("InsertionPoint", buf);
+
+  // rotation = geom_some(text->x_axis_dir);
+  // dtostring = doubletochar (rotation);
+  // newXMLProp ("Rotation", dtostring);
+
+  buf = inttochar (text->attachment);
+  newXMLProp ("AttachmentPoint", buf);
+  buf = inttochar (text->bg_fill_flag);
+  newXMLProp ("BackgroundFill", buf);
+  if (text->bg_fill_flag)
+    {
+      dtostring = doubletochar (text->bg_fill_scale);
+      newXMLProp ("BackgroundFill", dtostring);
+    }
+  // if (text->bg_fill_flag == 1)
+  //   {
+  //     buf = colortochar (text->bg_fill_color);
+  //     newXMLProp ("BackgroundColor", buf);
+  //   }
+  buf = inttochar (text->flow_dir);
+  newXMLProp ("DrawingDirection", buf);
+  dtostring = doubletochar (text->text_height);
+  newXMLProp ("LineSpacingDistance", dtostring);
+  dtostring = doubletochar (text->linespace_factor);
+  newXMLProp ("LineSpacingFactor", dtostring);
+  buf = inttochar (text->linespace_style);
+  newXMLProp ("LineSpacingStyle", buf);
+
+  newXMLcProp ("TextString", (xmlChar *)text->text);
+
+  name = dwg_ref_get_table_name (text->style, &error);
+  if (!error)
+    {
+      newXMLcProp ("StyleName", name);
+      if (dwg_version >= R_2007)
+        free (name);
+    }
+
+  dtostring = doubletochar (text->rect_width);
+  newXMLProp ("Width", dtostring);
 
   common_entity_attrs (node, obj);
   xmlAddChild (rootnode, node);
@@ -836,9 +936,9 @@ add_table (xmlNodePtr rootnode, const Dwg_Object *obj)
 
   newXMLcProp ("type", "IAcadTable");
   newXMLcProp ("desc", "IAcadTable: IAcadTable Interface");
+  newXMLcProp ("ObjectName", "AcDbTable");
 
-  buf = spointprepare (table->ins_pt.x, table->ins_pt.y,
-                       table->ins_pt.z);
+  buf = spointprepare (table->ins_pt.x, table->ins_pt.y, table->ins_pt.z);
   newXMLProp ("InsertionPoint", buf);
 
   dtostring = doubletochar (table->num_rows);
@@ -852,7 +952,7 @@ add_table (xmlNodePtr rootnode, const Dwg_Object *obj)
 
 // Main function
 int
-load_dwg (char *dwgfilename, xmlNodePtr rootnode)
+xml_dwg (char *dwgfilename, xmlNodePtr rootnode)
 {
   BITCODE_BL i;
   int error;
@@ -863,8 +963,11 @@ load_dwg (char *dwgfilename, xmlNodePtr rootnode)
   // Read the DWG file
   dwg.opts = 0; // silently
   error = dwg_read_file (dwgfilename, &dwg);
-  if (error)
-    return error;
+  if (error >= DWG_ERR_CRITICAL)
+    {
+      dwg_free (&dwg);
+      return error;
+    }
 
   // Emit some entities/objects to the XML file
   for (i = 0; i < dwg.num_objects; i++)
@@ -894,7 +997,7 @@ load_dwg (char *dwgfilename, xmlNodePtr rootnode)
           break;
 
         case DWG_TYPE_BLOCK:
-          //add_block (rootnode, obj);
+          // add_block (rootnode, obj);
           break;
 
         case DWG_TYPE_INSERT:
@@ -925,6 +1028,10 @@ load_dwg (char *dwgfilename, xmlNodePtr rootnode)
           add_text (rootnode, obj);
           break;
 
+        case DWG_TYPE_MTEXT:
+          add_text (rootnode, obj);
+          break;
+
         case DWG_TYPE_SPLINE:
           add_spline (rootnode, obj);
           break;
@@ -933,9 +1040,9 @@ load_dwg (char *dwgfilename, xmlNodePtr rootnode)
           add_xline (rootnode, obj);
           break;
 
-        /*case DWG_TYPE_TABLE:
-          add_table(rootnode, obj);
-          break;*/
+          /*case DWG_TYPE_TABLE:
+            add_table(rootnode, obj);
+            break;*/
 
         default:
           if (obj->type < 500 || (obj->type - 500) >= dwg.num_classes)
@@ -978,20 +1085,20 @@ main (int argc, char *argv[])
 
   if (root == NULL)
     {
-      printf ("Cannot get handle to root node. Please Check\n");
+      printf ("Cannot get handle to root node. Please check\n");
       return 1;
     }
 
-  // Load the DWG file
-  error = load_dwg (argv[1], root);
-  if (error)
+  // Process the DWG file
+  error = xml_dwg (argv[1], root);
+  if (error >= DWG_ERR_CRITICAL)
     {
       xmlFreeDoc (doc);
       xmlCleanupParser ();
       return error;
     }
 
-  // Save the XML and Cleanup
+  // Save the XML and cleanup
   xmlSaveFormatFileEnc (argv[2], doc, "UTF-8", 1);
   xmlFreeDoc (doc);
   xmlCleanupParser ();
